from io import RawIOBase
from typing import cast
from mcap.mcap0.stream_reader import StreamReader
from mcap_ros1.decoder import Decoder

from .generate import generate_sample_data


def test_ros_decoder_from_stream():
    with generate_sample_data() as m:
        stream_reader = StreamReader(cast(RawIOBase, m))
        ros_reader = Decoder(stream_reader)
        messages = [m for m in ros_reader.messages]
        assert len(messages) == 10


def test_ros_decoder_from_bytes():
    with generate_sample_data() as m:
        data = m.read()
        messages = [m for m in Decoder(data).messages]
        assert len(messages) == 10



from io import BytesIO, RawIOBase
from typing import Any, Dict, Union, cast

from genpy import dynamic  # type: ignore
from mcap.mcap0.exceptions import McapError
from mcap.mcap0.records import Channel, Message, Schema
from mcap.mcap0.stream_reader import StreamReader


class Decoder:
    def __init__(self, source: Union[bytes, StreamReader]):
        if isinstance(source, StreamReader):
            self.__reader = source
        else:
            self.__reader = StreamReader(cast(RawIOBase, BytesIO(source)))

    @property
    def messages(self):
        channels: Dict[int, Channel] = {}
        schemas: Dict[int, Schema] = {}
        msg_types: Dict[str, Any] = {}
        for record in self.__reader.records:
            if isinstance(record, Schema):
                schemas[record.id] = record
                if record.encoding != "ros1msg":
                    raise McapError(
                        f"Can't decode schema with encoding {record.encoding}"
                    )
                if not record.name in msg_types:
                    msg_type = dynamic.generate_dynamic(  # type: ignore
                        record.name, record.data.decode()
                    )
                    msg_types[record.name] = msg_type[record.name]
            if isinstance(record, Channel):
                channels[record.id] = record
            if isinstance(record, Message):
                channel = channels[record.channel_id]
                schema = schemas[channel.schema_id]
                message = msg_types[schema.name]().deserialize(record.data)
                yield (channel.topic, record, message)



import contextlib
import time
from io import BytesIO
from tempfile import TemporaryFile

from mcap.mcap0.writer import Writer
from std_msgs.msg import String  # type: ignore


@contextlib.contextmanager
def generate_sample_data():
    file = TemporaryFile("w+b")
    writer = Writer(file)
    writer.start(profile="ros1", library="test")
    string_schema_id = writer.register_schema(
        name=String._type, encoding="ros1msg", data=String._full_text.encode()  # type: ignore
    )
    string_channel_id = writer.register_channel(
        topic="chatter", message_encoding="ros1", schema_id=string_schema_id
    )

    for i in range(1, 11):
        s = String(data=f"string message {i}")
        buff = BytesIO()
        s.serialize(buff)  # type: ignore
        writer.add_message(
            channel_id=string_channel_id,
            log_time=time.time_ns(),
            data=buff.getvalue(),
            publish_time=time.time_ns(),
        )
    writer.finish()
    file.seek(0)

    yield file
